ODT_IMAGE = (
    '<draw:frame draw:name="img1" svg:width="{0}" svg:height="{1}" text:anchor-type="paragraph">'
    '<draw:image xlink:type="simple" xlink:show="embed" xlink:actuate="onLoad">'
    '<office:binary-data>{2}</office:binary-data>'
    '</draw:image></draw:frame>'
)



import base64

from django import template

from template_engines.odt_helpers import ODT_IMAGE
from django.utils.safestring import mark_safe

from .utils import resize

register = template.Library()


@register.simple_tag
def image_loader(image, i_width=None, i_height=None):
    """
    Replace a tag by an image you specified.
    You must add an entry to the ``context`` var that is a dict with at least a ``content`` key
    whose value is a byte object. You can also specify ``width`` and ``height``, otherwise it will
    automatically resize your image.
    """
    if isinstance(image, str):
        if image:
            image = {'content': base64.b64decode(image.split(';base64,')[1])}
        else:
            return
    width = i_width or image.get('width')
    height = i_height or image.get('height')
    content = image.get('content')

    width, height = resize(content, width, height)

    return mark_safe(ODT_IMAGE.format(width, height, base64.b64encode(content).decode()))



import re

from bs4 import BeautifulSoup
from django.conf import settings
from django.template import Context
from django.template.context import make_context

from . import ODT_PARAGRAPH_RE, TO_CHANGE_RE, ODT_CHANGES
from .abstract import AbstractTemplate, ZipAbstractEngine
from .utils import modify_libreoffice_doc


class OdtTemplate(AbstractTemplate):
    """
    Handles odt templates.
    """

    def __init__(self, template, template_path=None):
        """
        :param template: the template to fill.
        :type template: django.template.Template

        :param template_path: path to the template.
        :type template_path: str
        """
        super().__init__(template)
        self.template_path = template_path

    def clean(self, data):
        # Add bold style
        enriched_data = re.sub(
            '</office:automatic-styles>',
            (
                '<style:style style:name="BOLD" style:family="text">'
                + '<style:text-properties fo:font-weight="bold" style:font-weight-asian="bold"'
                + ' style:font-weight-complex="bold"/></style:style></office:automatic-styles>'
            ),
            data,
        )

        return ODT_PARAGRAPH_RE.sub(
            lambda e: TO_CHANGE_RE.sub(
                lambda x: ODT_CHANGES[x.group(0)].format(e.group(1)),
                e.group(0),
            ),
            enriched_data,
        )

    def replace_inputs(self, content):
        """ Replace all text:text-input to text-span """

        soup = BeautifulSoup(content, features='xml')
        input_list = soup.find_all("text:text-input")

        for tag in input_list:
            tag.name = 'span'
            tag.attrs = {}

        return soup.prettify()

    def get_escaped_var_value(self, value):
        """
        Encodes XML reserved chars in value (eg. &, <, >) and also replaces
        the control chars \n and \t control chars to their ODF counterparts.
        """
        return value.replace('\n', '<text:line-break/>')\
                    .replace('\t', '<text:tab/>')\
                    .replace('\x0b', '<text:space/>')\
                    .replace('\x0c', '<text:space/>')

    def render(self, context=None, request=None):
        context = make_context(context, request)
        rendered = self.template.render(Context(context))
        rendered = self.clean(rendered)
        rendered = self.replace_inputs(rendered)
        rendered = self.get_escaped_var_value(rendered)
        odt_content = modify_libreoffice_doc(self.template_path, 'content.xml', rendered)
        return odt_content


class OdtEngine(ZipAbstractEngine):
    """
    Odt template engine.

    By default, ``app_dirname`` is equal to 'templates' but you can change this value by adding an
    ``ODT_ENGINE_APP_DIRNAME`` setting in your settings.
    By default, ``sub_dirname`` is equal to 'odt' but you can change this value by adding an
    ``ODT_ENGINE_SUB_DIRNAME`` setting in your settings.
    By default, ``OdtTemplate`` is used as template_class.
    """
    sub_dirname = getattr(settings, 'ODT_ENGINE_SUB_DIRNAME', 'odt')
    app_dirname = getattr(settings, 'ODT_ENGINE_APP_DIRNAME', 'templates')
    template_class = OdtTemplate
    zip_root_file = 'content.xml'



import io
import zipfile
from os.path import join

from django.core.files.storage import default_storage
from django.template import Template
from django.template.backends.base import BaseEngine
from django.template.loader import TemplateDoesNotExist
from django.utils.functional import cached_property

from .utils import clean_tags


class AbstractTemplate:
    """
    Gives the architecture of a basic template.

    ``clean`` and ``render`` must be implemented.
    """

    def __init__(self, template):
        self.template = template

    def clean(self, data):
        """
        Method for cleaning rendered data.
        """
        raise NotImplementedError()

    def render(self, context=None, request=None):
        """
        Fills a template with the context obtained by combining the `context` and` request`
        parameters and returns a file as a byte object.
        """
        raise NotImplementedError()


class ZipAbstractEngine(BaseEngine):
    """
    Gives the architecture of a basic zip base template engine.

    Can be specified:

    * ``app_dirname``, the folder name which contains the templates in application directories,
    * ``sub_dirname``, the folder name of the subdirectory in the templates directory,
    * ``template_class``, your own template class with a ``render`` method,
    * ``zip_root_file``, the file to fill.
    """
    app_dirname = None
    sub_dirname = None
    template_class = None
    zip_root_file = None

    def __init__(self, params):
        params = params.copy()
        self.options = params.pop('OPTIONS')
        super().__init__(params)

    def get_template_content(self, filename):
        """
        Returns the contents of a template before modification, as a string.
        """
        try:
            template_buffer = io.BytesIO(default_storage.open(filename, 'rb').read())
            with zipfile.ZipFile(template_buffer, 'r') as zip_file:
                b_content = zip_file.read(self.zip_root_file)
                return b_content.decode()
        except KeyError:
            raise TemplateDoesNotExist('Bad format.')

    @cached_property
    def template_dirs(self):
        t_dirs = super().template_dirs
        if self.sub_dirname:
            t_dirs += tuple([join(p, self.sub_dirname) for p in t_dirs])
        return t_dirs

    def from_string(self, template_code, **kwargs):
        return self.template_class(Template(template_code), **kwargs)

    def get_template_path(self, filename):
        """
        Check if a template named ``template_name`` can be found in a list of directories. Returns
        the path if the file exists or raises ``TemplateDoesNotExist`` otherwise.
        """
        if default_storage.exists(filename):
            return filename
        for directory in self.template_dirs:
            abstract_path = default_storage.generate_filename(join(directory, filename))
            if default_storage.exists(abstract_path):
                return abstract_path
        raise TemplateDoesNotExist(f'Unknown: {filename}')

    def clean_content(self, content):
        return clean_tags(content)

    def get_template(self, template_name):
        template_path = self.get_template_path(template_name)
        content = self.get_template_content(template_path)
        content = self.clean_content(content)
        return self.from_string(content, template_path=template_path)
